﻿using System;
using System.Management;
using System.Diagnostics;
using System.Diagnostics.PerformanceData;
using System.Timers;
using System.IO;
using System.IO.Ports;
using System.Threading;
using System.Resources;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Numeric;
using System.Windows.Forms;
using Ivi.Driver.Interop;
using Agilent.AgM9392.Interop;





namespace MultiChannelStreamingDemo
{
    // Agilent Technologies Multichannel Example Program
    // Revision 1.0.0
    // Original Author: Dave Murray
    // Last Updated: 04/18/12

    // Requires Agilent M9392A software at version 1.2.0, Build 2383 or later.
    // Program should be run on a 64 bit operating system and built as a 64 bit application.
    // This is primarily required to allow the running process to acceess all the free system RAM
    // This program will build and run on a 32 bit OS but multichannel streaming capability will be severely limited.
    // Program can also be run without Hardware in Simulation Mode as long as the M9392A software is installed.

    

    public partial class Form1 : Form
    {


        bool bWaitForTrigger = false;
        bool exit = false;
        bool debug = true;
        

        double dSampleRate = 0;
        double frequency = 3e9;
        double power = -10;
        double bandwidth = 100e6;
        double acquisitionTime = 0.25;
        bool UsePreselector = false;
        int timeOut = 100000;
        double triggerDelay = 0;
        double TriggerLevel = 0.5;
        string Options = "";
        double SlaveTriggerLevel = -0.5;



        enum triggerSlope
        {
            POSITIVE = Agilent.AgM9392.Interop.AgM9392ExternalTriggerSlopeEnum.AgM9392ExternalTriggerSlopePositive,
            NEGATIVE = Agilent.AgM9392.Interop.AgM9392ExternalTriggerSlopeEnum.AgM9392ExternalTriggerSlopeNegative,
        }

        enum triggerMode
        {
            Ext = 0,
            Vid = 1,
            Imm = 2,
            Soft = 3,
        }

        enum recordingMode
        {
            AdjacentCh = 0,
            SameCh = 1,

        }

        short triggerModeSet = 0;
        short trigSlopeSet = 1;
        short recordMode = 1;

        Agilent.AgM9392.Interop.AgM9392 agDrvr1 = null;
        Agilent.AgM9392.Interop.AgM9392 agDrvr2 = null;


        string resourceDesc1 = null;
        string resourceDesc2 = null;


        string streamFile1 = "c:\\temp\\stream1.bin";//default
        string streamFile2 = "c:\\temp\\stream2.bin";//default



        bool FullVSAConfig = true;
        bool UsesRAID = false;
        bool UsingStreamingMode = true;
        bool UsesDigOnlyMode = false;

        bool IsSupportedCh1 = false;
        bool IsSupportedCh2 = false; 
        bool IsSimulate = true;
        bool UseFreeRAM = true;

        int NoOfChannels = 1;


        double CH1DiskSpaceGB = 0;
        double CH2DiskSpaceGB = 0;
        double FreeRAM_MB = 4000;// need non zero value for Init_VSA(s) 
        double TotalRAMReqMB = 0;
        double dDiskSpaceRequired = 0;
       
        double actualBW;

        int NoSamples;


        int ProgressCurrentPercent = 0;
        Color BarColor = Color.AliceBlue; // color of progress meter
        Graphics g;
        SolidBrush brush;
        System.Windows.Forms.Timer Timer1 = new System.Windows.Forms.Timer();// Not used at this time


        // Serial port 

        static SerialPort _serialPort = new SerialPort();
        bool SoftwareTrigger;


        Dictionary<string, string> dict = new Dictionary<string, string>(); // Required for File Config Load / Save

        List<string> ConfigData = new List<string>(); // Required for File Config Load / Save


        string LineItem = null;


        public Form1()
        {
            InitializeComponent();
            Load_Default_States();
            ConfigureSerialPort();
            Init_HW();
            InitializeConfigStates();
        }





        private void radioButton1_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void textBox4_TextChanged(object sender, EventArgs e)
        {

        }

        private void label1_Click(object sender, EventArgs e)
        {

        }

        private void label5_Click(object sender, EventArgs e)
        {

        }

        private void checkBox2_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void checkBox1_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void pictureBox1_Click(object sender, EventArgs e)
        {

        }

        private void Initialize_VSAs()
        {
            agDrvr1 = new Agilent.AgM9392.Interop.AgM9392();
            agDrvr2 = new Agilent.AgM9392.Interop.AgM9392();


            resourceDesc1 = Ch1_Resource.Text.ToString();
            resourceDesc2 = Ch2_Resource.Text.ToString();
            GetNoOfChannels();
            GetSimulateState();
            UpdateGlobalVariables();
            CH1StorageSpaceGB();
            CH2StorageSpaceGB();
            FreeRAM_MB = GetFreeMemoryMB();
            string Error = null;

            try
            {
                if (NoOfChannels == 1)  // 1 channel
                {
                    UpdateStatus.AppendText("Initializing Hardware .. Channel 1 ...\n");
                    agDrvr1.Initialize(resourceDesc1, true, true, Options);
                }
                if (NoOfChannels == 2)  // 2 channel
                {
                    UpdateStatus.AppendText("Initializing Hardware .. Channel 1 ...\n");//callback
                    agDrvr1.Initialize(resourceDesc1, true, true, Options);
                    UpdateStatus.AppendText("Initializing Hardware .. Channel 2 ...\n");//callback
                    agDrvr2.Initialize(resourceDesc2, true, true, Options);
                }
            }
            catch (Exception Err)
            {
                Error = Err.Message.ToString();
                UpdateStatus.AppendText(Error);
            }

           
            agDrvr1.Acquisition.PreviewCharacteristics(frequency, bandwidth, acquisitionTime, ref actualBW, ref dSampleRate, ref NoSamples);
            TotalRAMReqMB = Calculate_Required_RAM(dSampleRate, acquisitionTime, triggerDelay, NoOfChannels);
            ResourceManagement();

            UpdateStatus.AppendText("Initialization .. Done .. \n\n");//callback
            UpdateStatus.AppendText("Ready to configure your measurement.\n\n");//callback


        }

        private void IsStreamingSupported()
        {
            IsSupportedCh1 = agDrvr1.Streaming.IsSupported;
            IsSupportedCh2 = agDrvr2.Streaming.IsSupported;
        }

        private void Set_Streaming_Mode()
        {

        }

        private void Set_Trigger_Mode()
        {

        }

        private void Set_Measurement_Params()
        {

        }

        private void Set_Sweep_Mode()
        {

        }

        private int Take_Sweep()
        {
            return 0;
        }

        private double[] Get_IQ_Data()
        {
            double[] localIQData = null;
            return localIQData;
        }

        private void Graph_Data(double[] mIQData)
        {

        }

        private int Set_No_Of_Channels()
        {
            int iNoOfChannels = 0;
            
            if (checkedListBox4_No_Of_Chans.GetItemChecked(0))  // 1 channel
            {
                iNoOfChannels = 1;
            }
            else if (checkedListBox4_No_Of_Chans.GetItemChecked(1))  // 2 channel
            {
                iNoOfChannels = 2;
            }
            NoOfChannels = iNoOfChannels;
            return iNoOfChannels;
        }

        private void GetNoOfChannels()
        {
            if (checkedListBox4_No_Of_Chans.GetItemChecked(0))
                NoOfChannels = 1;
            else if (checkedListBox4_No_Of_Chans.GetItemChecked(1))
                NoOfChannels = 2;

        }

        private void CH1StorageSpaceGB()
        {
            string Letter = null;
            Letter = streamFile1[0].ToString();
            CH1DiskSpaceGB = GetFreeDiskSpace(Letter) / 1e9;
            string CH1DiskSpace = Convert.ToString(CH1DiskSpaceGB);
            textBox1CH1_Storage.Text = CH1DiskSpace;


        }

        private void CH2StorageSpaceGB()
        {
            string Letter = null;
            Letter = streamFile2[0].ToString();
            CH2DiskSpaceGB = GetFreeDiskSpace(Letter) / 1e9;
            string CH2DiskSpace = Convert.ToString(CH2DiskSpaceGB);
            textBox2CH2_Storage.Text = CH2DiskSpace;

        }

        private double Calculate_Required_RAM(double dSampleRate, double dAcquisitionTime, double PreTrigger, int iNoOfChannels)
        {
            double dTotalMemReq = 0;
            if (!UsesRAID)
            {
                dTotalMemReq = (500e6 + dSampleRate * 4 * dAcquisitionTime * iNoOfChannels)+ (dSampleRate * 4 * iNoOfChannels * (-1*PreTrigger));// add 500MB buffer
                dTotalMemReq = dTotalMemReq / 1e6; //MB Required
            }
            else
            {
                if (bandwidth > 20e6)
                {
                    dTotalMemReq = (500e6 + dSampleRate * 4 * iNoOfChannels) + (dSampleRate * 4 * iNoOfChannels * (-1*PreTrigger));// Add 500MB buffer / acquisition time not important for RAID
                    dTotalMemReq = dTotalMemReq / 1e6; //MB Required 
                }
                else
                {
                    dTotalMemReq = (250e6 + dSampleRate * 4 * iNoOfChannels) + (dSampleRate * 4 * iNoOfChannels * (-1*PreTrigger));// Add 250MB Buffer
                    dTotalMemReq = dTotalMemReq / 1e6; //MB Required
                }
            }
            string sTotalMemReq = Convert.ToString(dTotalMemReq);
            textBox1RAMRequired.Text = sTotalMemReq;
            return dTotalMemReq;
        }

        private double[] Allocate_Free_RAM()
        {
            double[] MemPerChan = null;
            ;
            return MemPerChan;
        }

       

        private bool Is_Hiband()
        {
            bool bIsHi = false;
            double dFreq = 0;

            dFreq = Convert.ToDouble(numericUpDown1FrequencyMHz.Value);
            if (dFreq >= 2750)
                bIsHi = true;
            else
                bIsHi = false;

            return bIsHi;
        }

        private bool Uses_Preselector()
        {
            return UsePreselector;
        }

        private bool Uses_RAID()
        {
            return UsesRAID;
        }

        private bool Uses_VSAs()
        {
            return FullVSAConfig;
        }



        private void Display_Message(string message)
        {

        }

        private void Configure_Progress_Bar()
        {

        }

        private void Update_Progress_Bar()
        {

        }

        private int Are_We_Done()
        {
            return 0;
        }

        private void softwareToolStripMenuItem_Click(object sender, EventArgs e)
        {
            int index = 0;
            string Manf = null;
            string Instr = null;
            string FW = null;
            string Ser = null;

            UpdateStatus.AppendText("\n\n");

            if (NoOfChannels == 1)
            {
                index = 0;
                index = agDrvr1.System.ModuleCount;
                for (int i = 0; i <= index - 1; i++)
                {
                    agDrvr1.System.GetModuleIdentity(i, ref Manf, ref Instr, ref FW, ref Ser);
                    string line = "VSA1::" + Manf + " ; " + Instr + " ; " + FW + " ; " + Ser + "\n";
                    UpdateStatus.AppendText(line);
                }
                UpdateStatus.AppendText("\n\n");
            }
            if (NoOfChannels == 2)
            {
                index = 0;
                index = agDrvr1.System.ModuleCount;
                for (int i = 0; i <= index - 1; i++)
                {
                    agDrvr1.System.GetModuleIdentity(i, ref Manf, ref Instr, ref FW, ref Ser);
                    string line = "VSA1::" + Manf + " ; " + Instr + " ; " + FW + " ; " + Ser + "\n";
                    UpdateStatus.AppendText(line);
                }
                UpdateStatus.AppendText("\n");
                index = agDrvr2.System.ModuleCount;
                for (int i = 0; i <= index - 1; i++)
                {
                    agDrvr2.System.GetModuleIdentity(i, ref Manf, ref Instr, ref FW, ref Ser);
                    string line = "VSA2::" + Manf + " ; " + Instr + " ; " + FW + " ; " + Ser + "\n";
                    UpdateStatus.AppendText(line);
                }
                UpdateStatus.AppendText("\n\n");
            }

        }

        private void openFileDialog1_FileOk(object sender, CancelEventArgs e)
        {
            OpenFileDialog OpenFileDialog1 = new OpenFileDialog();
            OpenFileDialog1.Filter = "Streaming Config|*.txt";
            OpenFileDialog1.Title = "Open a Config File";
            OpenFileDialog1.ShowDialog();

            // If the file name is not an empty string open it for saving.
            if (OpenFileDialog1.FileName != "")
            {
                // Saves the record via a FileStream created by the OpenFile method.
                System.IO.FileStream fs =
                   (System.IO.FileStream)OpenFileDialog1.OpenFile();


                fs.Close();
            }
        }


        private void textBox5_TextChanged(object sender, EventArgs e)
        {

        }

        private void Update_Form_States()
        {

        }

        private void Load_Default_States()
        {
            agDrvr1 = new Agilent.AgM9392.Interop.AgM9392();
            agDrvr2 = new Agilent.AgM9392.Interop.AgM9392();

            NoOfChannels = 1;

            checkBox1_Simulate.Checked = true;// default Simulate
            checkedListBox4_No_Of_Chans.SetItemChecked(0, true);// 1 CH
            checkedListBox3_HW_Config.SetItemChecked(0, true);//FullVSA(s)
            checkedListBox2_Storage.SetItemChecked(0, true);//RAID
            checkedListBox1_TriggerMode.SetItemChecked(2, true);// Imm Trigger
            checkedListBox6TriggerSlope.SetItemChecked(0, true);// pos
            checkedListBox1RecordMode.SetItemChecked(1, true);// SameCh

            UpdateGlobalVariables();


            UpdateStatus.AppendText("Multichannel Streaming Example Program\n\n");//callback
            UpdateStatus.ScrollBars = RichTextBoxScrollBars.ForcedVertical;



        }

        private void CH1_Resource_TextChanged(object sender, EventArgs e)
        {
            resourceDesc1 = Ch1_Resource.Text.Trim().ToString();

        }

        private void Get_IVIResources()
        {
            resourceDesc1 = Ch1_Resource.Text.Trim().ToString();
            resourceDesc2 = Ch2_Resource.Text.Trim().ToString();
        }

        private void checkedListBox1_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (checkedListBox1_TriggerMode.SelectedIndex == 0)
            {
                checkedListBox1_TriggerMode.SetItemChecked(1, false);//toggle state
                checkedListBox1_TriggerMode.SetItemChecked(2, false);//toggle state
                numericUpDown2TriggerLevel.Value = 1; // default Ext trigger level

            }
            if (checkedListBox1_TriggerMode.SelectedIndex == 1)
            {
                checkedListBox1_TriggerMode.SetItemChecked(0, false);//toggle state
                checkedListBox1_TriggerMode.SetItemChecked(2, false);//toggle state
                numericUpDown2TriggerLevel.Value = -20; // default Vid trigger level
            }

           

            if (checkedListBox1_TriggerMode.SelectedIndex == 2)
            {
                if (NoOfChannels == 2)
                {
                    UpdateStatus.AppendText("Unsupported Trigger Mode for Multichannel! Setting to Video Trigger.\n");//callback
                    checkedListBox1_TriggerMode.SetItemChecked(1, true);// force Vid Trigger
                    checkedListBox1_TriggerMode.SetItemChecked(0, false);//toggle state
                    checkedListBox1_TriggerMode.SetItemChecked(2, false);//toggle state
                }
                else
                {
                    checkedListBox1_TriggerMode.SetItemChecked(0, false);//toggle state
                    checkedListBox1_TriggerMode.SetItemChecked(1, false);//toggle state
                }
            }



            GetTriggerMode();



        }

        private void CheckTriggerMode()
        {

            if (checkedListBox1_TriggerMode.GetItemChecked(2))
            {
                if (NoOfChannels == 2)
                {
                    UpdateStatus.AppendText("Unsupported Trigger Mode for Multichannel! Setting to Video Trigger\n");//callback
                    checkedListBox1_TriggerMode.SetItemChecked(1, true);// force Video Trigger
                    checkedListBox1_TriggerMode.SetItemChecked(0, false);//toggle state
                    checkedListBox1_TriggerMode.SetItemChecked(2, false);//toggle state
                    UpdateStatus.AppendText("Setting to Video Trigger to default level\n");//callback
                    numericUpDown2TriggerLevel.Value = -20; // set default trigger level
                }
                else
                {
                    checkedListBox1_TriggerMode.SetItemChecked(0, false);//toggle state
                    checkedListBox1_TriggerMode.SetItemChecked(1, false);//toggle state
                }
            }
            GetTriggerMode();
        }

        private void GetTriggerMode()
        {
            if (checkedListBox1_TriggerMode.GetItemChecked(0))
                triggerModeSet = Convert.ToInt16(triggerMode.Ext);
            else if (checkedListBox1_TriggerMode.GetItemChecked(1))
                triggerModeSet = Convert.ToInt16(triggerMode.Vid);
            else if (checkedListBox1_TriggerMode.GetItemChecked(2))
                triggerModeSet = Convert.ToInt16(triggerMode.Imm);

            if (radioButton2SerialPortTrigger.Checked)
            {
                triggerModeSet = Convert.ToInt16(triggerMode.Ext);
                checkedListBox1_TriggerMode.SetSelected(0, true);
            }
        }

        private void checkedListBox2_SelectedIndexChanged(object sender, EventArgs e)
        {
           
                    if (checkedListBox2_Storage.SelectedIndex == 0)
                    {
                        checkedListBox2_Storage.SetItemChecked(1, false);//toggle state
                        UsesRAID = true;
                        UpdateStatus.AppendText("Only minimal RAM dependancy when RAID is used -- Except if using Pre-trigger!\n");//callback

                    }
                    if (checkedListBox2_Storage.SelectedIndex == 1)
                    {
                        checkedListBox2_Storage.SetItemChecked(0, false);//toggle state
                        UsesRAID = false;
                        UpdateStatus.AppendText("RAM Dependant acquisition with only Single Hard Disk!\n");//callback

                    }


                    ResourceManagement();
               
            
        }

        private void GetStorageConfig()
        {
            if (checkedListBox2_Storage.GetItemChecked(0))
                UsesRAID = true;
            else if (checkedListBox2_Storage.GetItemChecked(1))
                UsesRAID = false;
        }


        private void button4SerialPortTrigger_Click(object sender, EventArgs e)
        {
            if (SoftwareTrigger)
            {
                SendSerialPortTrigger(); // Not used at this time
                UpdateStatus.AppendText("Software Trigger Sent\n");
            }
        }

        private void checkBox5_CheckedChanged(object sender, EventArgs e)
        {

        }

        private void numericUpDown2_ValueChanged(object sender, EventArgs e)
        {
            bandwidth = Convert.ToDouble(numericUpDown2BandwidthMHz.Value) * 1e6;
            agDrvr1.Acquisition.PreviewCharacteristics(frequency, bandwidth, acquisitionTime, ref actualBW, ref dSampleRate, ref NoSamples);
            ResourceManagement();
            GetPreselector();
        }

        private void GetBandWidth()
        {
            bandwidth = Convert.ToDouble(numericUpDown2BandwidthMHz.Value) * 1e6;
        }



        private void Form1_Load(object sender, EventArgs e)
        {

        }




        private void UpdateStatus_TextChanged(object sender, EventArgs e)
        {
            UpdateStatus.ScrollToCaret();
        }

        private void Free_VSA_Resources()
        {

            agDrvr1.Close();
            agDrvr2.Close();

        }

        private void checkedListBox4_No_Of_Chans_SelectedIndexChanged(object sender, EventArgs e)
        {
           
                    UpdateStatus.AppendText("Changing HW Configuration...Please Wait!\n");

                    if (checkedListBox4_No_Of_Chans.SelectedIndex == 0)
                    {
                        checkedListBox4_No_Of_Chans.SetItemChecked(1, false);//toggle state
                        checkedListBox4_No_Of_Chans.SetItemChecked(0, true);//toggle state

                    }
                    if (checkedListBox4_No_Of_Chans.SelectedIndex == 1)
                    {
                        checkedListBox4_No_Of_Chans.SetItemChecked(0, false);//toggle state
                        checkedListBox4_No_Of_Chans.SetItemChecked(1, true);//toggle state

                    }
                    NoOfChannels = Set_No_Of_Channels();
                    ResourceManagement();

                    UpdateStatus.AppendText("Closing Driver Sessions!\n");
                    string Error = null;

                    try
                    {

                        if (agDrvr1.Initialized)
                        {
                            if (NoOfChannels == 1)
                            {
                                agDrvr1.Measurements.Abort();
                                agDrvr1.Close();
                            }
                            else
                            {
                                agDrvr1.Close();

                                if (agDrvr2.Initialized)
                                {
                                    agDrvr2.Measurements.Abort();
                                    agDrvr2.Close();
                                }
                            }
                        }
                    }
                    catch (Exception err)
                    {
                        Error = err.Message.ToString();
                        UpdateStatus.AppendText(Error);
                    }


                    try
                    {

                        if (NoOfChannels == 1)
                        {
                            UpdateStatus.AppendText("Open Driver Session!\n");
                            agDrvr1 = new Agilent.AgM9392.Interop.AgM9392();
                        }
                        else
                        {
                            UpdateStatus.AppendText("Open Driver Sessions!\n");
                            agDrvr1 = new Agilent.AgM9392.Interop.AgM9392();
                            agDrvr2 = new Agilent.AgM9392.Interop.AgM9392();
                        }
                    }
                    catch (Exception err)
                    {
                        Error = err.Message.ToString();
                        UpdateStatus.AppendText(Error);
                    }


                    UpdateGlobalVariables();



                    try
                    {

                        if (NoOfChannels == 1)  // 1 channel
                        {
                            UpdateStatus.AppendText("Initializing Hardware .. Channel 1 ...\n");
                            agDrvr1.Initialize(resourceDesc1, true, true, Options);
                            UpdateStatus.AppendText("Done!\n");
                        }
                        if (NoOfChannels == 2)  // 2 channel
                        {
                            UpdateStatus.AppendText("Initializing Hardware .. Channel 1 ...\n");
                            agDrvr1.Initialize(resourceDesc1, true, true, Options);
                            UpdateStatus.AppendText("Done!\n");
                            UpdateStatus.AppendText("Initializing Hardware .. Channel 2 ...\n");
                            agDrvr2.Initialize(resourceDesc2, true, true, Options);
                            UpdateStatus.AppendText("Done!\n");
                        }
                    }
                    catch (Exception err)
                    {
                        Error = err.Message.ToString();
                        UpdateStatus.AppendText(Error);
                    }

                    try
                    {
                        agDrvr1.Acquisition.PreviewCharacteristics(frequency, bandwidth, acquisitionTime, ref actualBW, ref dSampleRate, ref NoSamples);
                    }
                    catch (Exception err)
                    {
                        Error = err.Message.ToString();
                        UpdateStatus.AppendText(Error);
                    }

                    ResourceManagement();
                }
               
    



        private void Change_HW_Configuration()
        {

            UpdateStatus.AppendText("Changing HW Configuration...Please Wait!\n");

            if (checkedListBox4_No_Of_Chans.SelectedIndex == 0)
            {
                checkedListBox4_No_Of_Chans.SetItemChecked(1, false);//toggle state
                checkedListBox4_No_Of_Chans.SetItemChecked(0, true);//toggle state

            }
            if (checkedListBox4_No_Of_Chans.SelectedIndex == 1)
            {
                checkedListBox4_No_Of_Chans.SetItemChecked(0, false);//toggle state
                checkedListBox4_No_Of_Chans.SetItemChecked(1, true);//toggle state

            }
            NoOfChannels = Set_No_Of_Channels();
            ResourceManagement();

            UpdateStatus.AppendText("Closing Driver Sessions!\n");//callback

            string Error = null;

             try
                {
                if (agDrvr1.Initialized)
                    {
                    if (NoOfChannels == 1)
                        {
                         agDrvr1.Measurements.Abort();
                         agDrvr1.Close();
                        }
                     else
                        {
                         agDrvr1.Measurements.Abort();
                         agDrvr1.Close();

                         if (agDrvr2.Initialized)
                            {
                            agDrvr2.Measurements.Abort();
                            agDrvr2.Close();
                            }
                        }
                    }
                }
            catch (Exception err)
                {
                    Error = err.Message.ToString();
                    UpdateStatus.AppendText(Error);
                }




                if (NoOfChannels == 1)
                {
                    agDrvr1 = new Agilent.AgM9392.Interop.AgM9392();
                }
                else
                {
                    UpdateStatus.AppendText("Open Driver Sessions!\n");
                    agDrvr1 = new Agilent.AgM9392.Interop.AgM9392();
                    agDrvr2 = new Agilent.AgM9392.Interop.AgM9392();
                }


                UpdateGlobalVariables();


                try
                {

                    if (NoOfChannels == 1)  // 1 channel
                    {
                        UpdateStatus.AppendText("Initializing Hardware .. Channel 1 ...\n");
                        agDrvr1.Initialize(resourceDesc1, true, true, Options);
                        UpdateStatus.AppendText("Done!\n");
                    }
                    if (NoOfChannels == 2)  // 2 channel
                    {
                        UpdateStatus.AppendText("Initializing Hardware .. Channel 1 ...\n");
                        agDrvr1.Initialize(resourceDesc1, true, true, Options);
                        UpdateStatus.AppendText("Done!\n");//callback
                        UpdateStatus.AppendText("Initializing Hardware .. Channel 2 ...\n");
                        agDrvr2.Initialize(resourceDesc2, true, true, Options);
                        UpdateStatus.AppendText("Done!\n");
                    }
                }
                catch (Exception err)
                {
                    Error = err.Message.ToString();
                    UpdateStatus.AppendText(Error);
                }

                try
                {
                    agDrvr1.Acquisition.PreviewCharacteristics(frequency, bandwidth, acquisitionTime, ref actualBW, ref dSampleRate, ref NoSamples);
                }
                catch (Exception err)
                {
                    Error = err.Message.ToString();
                    UpdateStatus.AppendText(Error);
                }

                ResourceManagement();
            
            
        }


        private void numericUpDown1FrequencyMHz_ValueChanged(object sender, EventArgs e)
        {
            frequency = Convert.ToDouble(numericUpDown1FrequencyMHz.Value) * 1e6;
            GetPreselector();
        }

        private void GetFrequency()
        {
            frequency = Convert.ToDouble(numericUpDown1FrequencyMHz.Value) * 1e6;

        }

        private bool Is_WideEnough()
        {
            bool IsWideEnough;

            if (bandwidth > 40e6)
            {
                IsWideEnough = false;
            }
            else
                IsWideEnough = true;

            return IsWideEnough;
        }


        private void GetPreselector()
        {
            bool bState;
            bool bWide;
            bState = Is_Hiband();
            bWide = Is_WideEnough();

            if (!bState || !bWide)
            {

                if (!bState && checkBox1UsePreselector.Checked)
                {
                    UpdateStatus.AppendText("No Preselector below 2750 MHz!\n");
                }
                if (!bWide && checkBox1UsePreselector.Checked)
                {
                    UpdateStatus.AppendText("Selected Bandwidth too wide for Preselector - 40 MHz Max Allowed!\n");
                }
                checkBox1UsePreselector.Checked = false;
            }

            if (checkBox1UsePreselector.Checked)
                UsePreselector = true;
            else
                UsePreselector = false;
        }

        private void Ch2_Resource_TextChanged(object sender, EventArgs e)
        {
            resourceDesc2 = Ch2_Resource.Text.Trim().ToString();

        }



        private void textBox1StreamName1_TextChanged(object sender, EventArgs e)
        {


            streamFile1 = textBox1StreamName1.Text.Trim().ToString();
            CH1StorageSpaceGB();
        }

        private void textBox2StreamName2_TextChanged(object sender, EventArgs e)
        {
            streamFile2 = textBox2StreamName2.Text.Trim().ToString();
            CH2StorageSpaceGB();
        }

        private void GetStreamingFiles()
        {
            streamFile1 = textBox1StreamName1.Text.Trim().ToString();
            CH1StorageSpaceGB();
            streamFile2 = textBox2StreamName2.Text.Trim().ToString();
            CH2StorageSpaceGB();
        }

        private bool CheckVolume()
        {
            bool bSameVolume = false;
            string[] s1, s2;
            s1 = streamFile1.Split(':');
            s2 = streamFile2.Split(':');
            if (s1[0] == s2[0])
                bSameVolume = true;
            else
                bSameVolume = false;

            return bSameVolume;
        }






        private void checkedListBox6TriggerSlope_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (checkedListBox6TriggerSlope.SelectedIndex == 0)
            {
                checkedListBox6TriggerSlope.SetItemChecked(1, false);//toggle state
                checkedListBox6TriggerSlope.SetItemChecked(0, true);//toggle state
            }
            if (checkedListBox6TriggerSlope.SelectedIndex == 1)
            {
                checkedListBox6TriggerSlope.SetItemChecked(0, false);//toggle state
                checkedListBox6TriggerSlope.SetItemChecked(1, true);//toggle state
            }

            GetTriggerSlope();
        }

        private void GetTriggerSlope()
        {
            if (checkedListBox6TriggerSlope.GetItemChecked(0))
                trigSlopeSet = Convert.ToInt16(triggerSlope.POSITIVE);
            else if (checkedListBox6TriggerSlope.GetItemChecked(1))
                trigSlopeSet = Convert.ToInt16(triggerSlope.NEGATIVE);
        }

        private void radioButton3DigitizerOnlyMode_CheckedChanged(object sender, EventArgs e)
        {
            UpdateStatus.AppendText("Digitizer Only Mode [Debug]---- Not Available!\n");
            UsesDigOnlyMode = radioButton3DigitizerOnlyMode.Checked;
        }

        private void GetDigitizerOnlyState()
        {
            UsesDigOnlyMode = radioButton3DigitizerOnlyMode.Checked;
        }




        private void checkedListBox1RecordMode_SelectedIndexChanged(object sender, EventArgs e)
        {
            
                    if (checkedListBox1RecordMode.SelectedIndex == 0)
                    {
                        checkedListBox1RecordMode.SetItemChecked(1, false);//toggle state
                    }
                    if (checkedListBox1RecordMode.SelectedIndex == 1)
                    {
                        checkedListBox1RecordMode.SetItemChecked(0, false);//toggle state
                    }

                    if (checkedListBox1RecordMode.GetItemChecked(0))
                    {
                        if (FullVSAConfig && NoOfChannels==2)
                        {
                            recordMode = Convert.ToInt16(recordingMode.AdjacentCh);
                            UpdateStatus.AppendText("Adjacent Channel [Independant LO] Mode Set!\n");
                            UpdateStatus.AppendText("Slave VSA will be offset by  + Bandwidth/2!\n");
                        }
                       else
                        {
                            UpdateStatus.AppendText("Adjacent Channel [Independant LO] Mode Not Supported!\n");
                            checkedListBox1RecordMode.SetItemChecked(0, false);//toggle state
                            checkedListBox1RecordMode.SetItemChecked(1, true);//toggle state
                            UpdateStatus.AppendText("Same Channel [Same LO] Mode Set!\n");
                            recordMode = Convert.ToInt16(recordingMode.SameCh);

                        }
                    }

                    else if (checkedListBox1RecordMode.GetItemChecked(1))
                    {
                        recordMode = Convert.ToInt16(recordingMode.SameCh);
                        UpdateStatus.AppendText("Same Channel [Same LO] Mode Set!\n");
                    }
                
               
        }

        private void GetRecordingMode()
        {
            if (checkedListBox1RecordMode.GetItemChecked(0))
                recordMode = Convert.ToInt16(recordingMode.AdjacentCh);

            else if (checkedListBox1RecordMode.GetItemChecked(1))
                recordMode = Convert.ToInt16(recordingMode.SameCh);
        }

        private void CheckRecordingMode()
        {
            if (checkedListBox1RecordMode.GetItemChecked(0))
            {
                Get_HW_Config();
                if (FullVSAConfig)
                {
                    recordMode = Convert.ToInt16(recordingMode.AdjacentCh);
                    UpdateStatus.AppendText("Adjacent Channel [Independant LO] Mode Set!\n");
                }
                else
                {
                    if (NoOfChannels == 2)
                    {
                        UpdateStatus.AppendText("Adjacent Channel [Independant LO] Mode Not Supported!\n");
                        checkedListBox1RecordMode.SetItemChecked(0, false);//toggle state
                        checkedListBox1RecordMode.SetItemChecked(1, true);//toggle state
                        UpdateStatus.AppendText("Same Channel [Same LO] Mode Set!\n");
                        recordMode = Convert.ToInt16(recordingMode.SameCh);
                    }
                }
            }
            else if (checkedListBox1RecordMode.GetItemChecked(1))
            {
                recordMode = Convert.ToInt16(recordingMode.SameCh);
                UpdateStatus.AppendText("Same Channel [Same LO] Mode Set!\n");
            }
}




        private void button2_Stop_Click(object sender, EventArgs e) // Not Used - Needs Multithreaded program
        {

            Thread.Sleep(100);

            bWaitForTrigger = false;
            try
            {
                if (UsingStreamingMode)
                {

                    //Stop Streaming
                    if (agDrvr1.Streaming.IsActive)
                    {
                        this.Timer1.Enabled = false;
                        UpdateStatus.AppendText("Current Recording Stopped...\n");
                        UpdateStatus.AppendText("Writing Buffer to Disk...\n");
                        exit = true;// boolean for abort wait trigger loop
                    }
                    if (NoOfChannels == 1)
                    {
                        agDrvr1.Streaming.Stop();
                    }
                    else if (NoOfChannels == 2)
                    {
                        agDrvr1.Streaming.Stop();
                        agDrvr2.Streaming.Stop();
                    }

                }
                else
                {
                    if (agDrvr1.Streaming.IsActive)
                    {
                        this.Timer1.Enabled = false; // Not Used
                        UpdateStatus.AppendText("Current Acquisition Stopped...\n");
                    }
                    if (NoOfChannels == 1)
                    {
                        agDrvr1.Measurements.Abort();
                    }
                    else if (NoOfChannels == 2)
                    {
                        agDrvr1.Measurements.Abort();
                        agDrvr2.Measurements.Abort();
                    }


                    //Stop acquiring data on selected channel
                   
                }
            }
            catch (Exception err)
            {
                string Error = null;
                Error = err.Message.ToString();
                UpdateStatus.AppendText(Error);

            }
            finally
            {
                if (NoOfChannels == 1)
                {
                    if (agDrvr1.Streaming.IsActive)
                        agDrvr1.Measurements.Abort();
                }
                else if (NoOfChannels == 2)
                {
                    if (agDrvr1.Streaming.IsActive)
                        agDrvr1.Measurements.Abort();
                    if (agDrvr2.Streaming.IsActive)
                        agDrvr2.Measurements.Abort();
                }
                progressBar1.Value = 0;
                
            }
        }

        private void button3_Abort_Click(object sender, EventArgs e)  // Not Used - Needs Multithreaded Program
        {
            Thread.Sleep(100);

            bWaitForTrigger = false;

            try
            {
                if (UsingStreamingMode)
                {
                    if (agDrvr1.Streaming.IsActive)
                    {
                        this.Timer1.Enabled = false;
                        UpdateStatus.AppendText("Streaming Acquisition Aborted...\n");
                        //Abort Streaming
                        if (NoOfChannels == 1)
                            agDrvr1.Measurements.Abort();
                        else if (NoOfChannels == 2)
                        {
                            agDrvr1.Measurements.Abort();
                            agDrvr2.Measurements.Abort();
                        }
                        exit = true;// boolean for abort wait trigger loop
                    }
                }
                else
                {
                    UpdateStatus.AppendText("Acquisition Stopped...\n");//callback
                    //abort acquisition on selected channel
                    if (NoOfChannels == 1)
                    {
                        agDrvr1.Measurements.Abort();
                    }
                    else if (NoOfChannels == 2)
                    {
                        agDrvr1.Measurements.Abort();
                        agDrvr2.Measurements.Abort();
                    }
                }
            }
            catch (Exception err)
            {
                string Error = null;
                Error = err.Message.ToString();
                UpdateStatus.AppendText(Error);
            }
            finally
            {
                if (NoOfChannels == 1)
                {
                    if (agDrvr1.Streaming.IsActive)
                        agDrvr1.Measurements.Abort();
                }
                else if (NoOfChannels == 2)
                {
                    if (agDrvr1.Streaming.IsActive)
                        agDrvr1.Measurements.Abort();
                    if (agDrvr2.Streaming.IsActive)
                        agDrvr2.Measurements.Abort();
                }

                progressBar1.Value = 0;
                
            }

        }

        public static string SetPortName(string defaultPortName)
        {
            string portName;
            portName = defaultPortName;
            return portName;
        }

        private void ConfigureSerialPort()
        {

            _serialPort.PortName = "COM1";
            _serialPort.BaudRate = 9600;
            _serialPort.Parity = Parity.None;
            _serialPort.DataBits = 8;
            //_serialPort.StopBits = StopBits.None;
            _serialPort.Handshake = Handshake.None;
            _serialPort.RtsEnable = false;


        }

        private void SendSerialPortTrigger()    // Not Used at this time
        {

            _serialPort.Open(); //open serial port
            _serialPort.DtrEnable = true; // data terminal ready  pin 4 of DCE
            //_serialPort.WriteLine("Fire");// dummy data
            _serialPort.Close();// end serial communication

        }

        private void radioButton2SerialPortTrigger_CheckedChanged(object sender, EventArgs e)
        {
            if (radioButton2SerialPortTrigger.Checked)
            {
                SoftwareTrigger = true;
            }
            else
            {
                SoftwareTrigger = false;
            }

        }

        private void GetSerialPortTrigger()
        {
            SoftwareTrigger = radioButton2SerialPortTrigger.Checked;
        }

        private double GetFreeMemoryMB()
        {
            System.Diagnostics.PerformanceCounter ramCounter = new System.Diagnostics.PerformanceCounter("Memory", "Available KBytes");

            double _freeMemory = ramCounter.NextValue();

            _freeMemory = _freeMemory / 1e3;


            string FreeRam = Convert.ToString(_freeMemory);
            textBox3FreeRAM.Text = FreeRam;

            return _freeMemory;
        }

        private string GetDriveVolumeMaster()
        {
            string Letter;
            string[] Volume = streamFile1.Split(':');
            Letter = Volume[0];
            return Letter.Trim();
        }

        private string GetDriveVolumeSlave1()
        {
            string Letter;
            string[] Volume = streamFile2.Split(':');
            Letter = Volume[0];
            return Letter.Trim();
        }



        private double CalculateMemoryRequirement(double acquisitionTime, double SampleRate, double triggerDelay)
        {
            double RamRequired = acquisitionTime * SampleRate * 4 ; // # of Bytes
            return RamRequired;
        }

        private double CalculateDiskSpaceRequirement(double acquisitionTime, double SampleRate, double triggerDelay)
        {
            double DiskSpaceRequired = acquisitionTime * SampleRate * 4 ; // # of Bytes
            return DiskSpaceRequired;
        }

        private void SetMemoryGrabber(int size, int time)  // Memory in MB,  time in Millseconds.   NOT REQUIRED When RAM Buffer Hint Is Used
        {
            string sSize, sTime, sCommand;
            sSize = Convert.ToString(size);
            sTime = Convert.ToString(time);
            sCommand = "-m + sSize -w + sTime";

            System.Diagnostics.Process.Start("StreamUtil.exe", sCommand);
        }



        private double GetFreeDiskSpace(string Letter)
        {
            double freeDiskSpace = 0;
            try
            {
                System.IO.DriveInfo driveInfo = new System.IO.DriveInfo(Letter);
                freeDiskSpace = driveInfo.TotalFreeSpace;
            }
            catch (Exception ex)
            {
                UpdateStatus.AppendText(ex.Message.ToString());
            }
            finally
            {

            }
            return freeDiskSpace;
        }





        private void SingleChannelStreamingSetup()
        {
            if (UsingStreamingMode)
            {
                UpdateGlobalVariables();
                // setup trigger mode
                SetupTriggerMode();
                string Err = null;

                exit = false; //exit for streaming Acq - stop, abort, timeout

                //setup measurement
                agDrvr1.Acquisition.Frequency = frequency; //Hz
                agDrvr1.Acquisition.PreviewCharacteristics(frequency, bandwidth, acquisitionTime, ref actualBW, ref dSampleRate, ref NoSamples);
                agDrvr1.Acquisition.Bandwidth = bandwidth; //Hz
                agDrvr1.Acquisition.Power = power; //dBm
                agDrvr1.Acquisition.PreselectorEnabled = UsePreselector;// check valid
                agDrvr1.Acquisition.Time = acquisitionTime;


                if (NoOfChannels == 1)
                {
                    agDrvr1.Streaming.Enabled = true;
                    agDrvr1.Trigger.Delay = triggerDelay;
                    agDrvr1.Streaming.AcquisitionMode = AgM9392StreamingAcquisitionModeEnum.AgM9392StreamingAcquisitionModeDurationLimited;
                    agDrvr1.Streaming.FileFormat = AgM9392StreamingFileFormatEnum.AgM9392StreamingFileFormatN5110A;
                    agDrvr1.Streaming.Filename = streamFile1;
                    UpdateStatus.AppendText("Streaming Mode - 1 Channel \n");//callback

                }

                if (UseFreeRAM)
                {
                    UpdateStatus.AppendText("Sharing available Memory!\n");
                    Err = null;
                    double dLockMemory;


                    dLockMemory = FreeRAM_MB; // Set maximum RAM Buffer Size on Digitizer
                    dLockMemory = (uint)dLockMemory;// scale memory with pre-trigger requirements

                    try
                    {
                        agDrvr1.Acquisition.Hints.Add(AgM9392HintTypeEnum.AgM9392HintTypeStreamingMaxBufferSizeInBytes, (long)(dLockMemory * 1e6));

                    }
                    catch (Exception ex)
                    {
                        Err = ex.Message.ToString();
                        UpdateStatus.AppendText(Err);
                    }

                }
                else
                {
                    Err = null;
                    try
                    {
                        agDrvr1.Acquisition.Hints.Add(AgM9392HintTypeEnum.AgM9392HintTypeStreamingMaxBufferSizeInBytes, 200000000);
                    }
                    catch (Exception ex)
                    {
                        Err = ex.Message.ToString();
                        UpdateStatus.AppendText(Err);
                    }
                }


                try
                {
                    agDrvr1.Measurements.Initiate(); // required for streaming
                    agDrvr1.Streaming.Start(); // required for streaming

                }

                catch (Exception err)
                {
                    Err = err.Message.ToString();
                    UpdateStatus.AppendText(Err);

                }



                while (bWaitForTrigger)
                {
                    UpdateStatus.AppendText("Waiting For Trigger 1...\n");
                    CheckForTrigger();
                }
                this.Timer1.Enabled = true; // not used

                
                while (agDrvr1.Streaming.IsActive && !exit)
                {
                    StreamingStatusUpdate();
                    System.Threading.Thread.Sleep(500);

                }
               UpdateStatus.AppendText("Recording Complete\n");
            }
            this.Timer1.Enabled = false;
            progressBar1.Value = 0;
            
        }

        private void DualChannelStreamingSetup()
        {
            if (debug)
            {
                agDrvr1.Status.ErrorVerbosity = AgM9392ErrorVerbosityEnum.AgM9392ErrorVerbosityDefault;

            }


            // Setup Trigger Mode
            SetupMultichannelTriggerMode();
            UpdateGlobalVariables();



            exit = false; //exit for streaming Acq - stop, abort, timeout  Stop and Abort Not used in this example

            if (!FullVSAConfig)
            {
                frequency = 500e6;
                // Add control for M9352A, M9302A, M9168C and External MXG LO.
                // For 2 channel phase/time alignment capability will also need M9155C
            }


            if (UsingStreamingMode)
            {


                if (NoOfChannels == 2)
                {
                    IsStreamingSupported();
                    agDrvr1.Acquisition.Frequency = frequency; //Hz
                    agDrvr2.Acquisition.Frequency = frequency; //Hz
                    GetBandWidth();
                    agDrvr1.Acquisition.PreviewCharacteristics(frequency, bandwidth, acquisitionTime, ref actualBW, ref dSampleRate, ref NoSamples);
                    agDrvr1.Acquisition.Bandwidth = bandwidth; //Hz
                    agDrvr2.Acquisition.Bandwidth = bandwidth; //Hz


                    if (recordMode == Convert.ToInt16(recordingMode.AdjacentCh) && FullVSAConfig && NoOfChannels ==2)
                    {
                       
                        double sampleRate = dSampleRate;
                        //second channel must offset by Bandwidth  SampleRate/1.25
                        double dSlaveCenterFrequency = sampleRate / 1.25 + frequency; // offset by 1 bandwidth hi side
                        agDrvr2.Acquisition.Frequency = dSlaveCenterFrequency;
                    }


                    agDrvr1.Acquisition.Power = power; //dBm
                    agDrvr2.Acquisition.Power = power; //dBm
                    agDrvr1.Acquisition.Time = acquisitionTime;
                    agDrvr2.Acquisition.Time = acquisitionTime;
                    agDrvr1.Streaming.Enabled = true;
                    agDrvr2.Streaming.Enabled = true;
                    agDrvr1.Trigger.Delay = triggerDelay;
                    agDrvr2.Trigger.Delay = triggerDelay;
                    agDrvr1.Streaming.AcquisitionMode = AgM9392StreamingAcquisitionModeEnum.AgM9392StreamingAcquisitionModeDurationLimited;
                    agDrvr1.Streaming.FileFormat = AgM9392StreamingFileFormatEnum.AgM9392StreamingFileFormatN5110A;
                    agDrvr1.Streaming.Filename = streamFile1;
                    agDrvr2.Streaming.AcquisitionMode = AgM9392StreamingAcquisitionModeEnum.AgM9392StreamingAcquisitionModeDurationLimited;
                    agDrvr2.Streaming.FileFormat = AgM9392StreamingFileFormatEnum.AgM9392StreamingFileFormatN5110A;
                    agDrvr2.Streaming.Filename = streamFile2;
                    UpdateStatus.AppendText("Streaming Mode - 2 Channels \n");

                    DisplayErrors();


                    if (FullVSAConfig)
                    {
                        agDrvr1.Acquisition.PreselectorEnabled = UsePreselector;// check valid
                        agDrvr2.Acquisition.PreselectorEnabled = UsePreselector;// check valid
                    }

                    IsStreamingSupported();

                    if (UseFreeRAM)
                    {
                        UpdateStatus.AppendText("Sharing available Memory!\n");
                        string Err = null;
                        double dLockMemory;


                        dLockMemory = (FreeRAM_MB) / 2; // Set maximum RAM Buffer memory per Digitizer
                        dLockMemory = (uint)dLockMemory;// 

                        try
                        {
                            agDrvr1.Acquisition.Hints.Add(AgM9392HintTypeEnum.AgM9392HintTypeStreamingMaxBufferSizeInBytes, (long)(dLockMemory * 1e6));//Required
                            agDrvr2.Acquisition.Hints.Add(AgM9392HintTypeEnum.AgM9392HintTypeStreamingMaxBufferSizeInBytes, (long)(dLockMemory * 1e6));//Required
                        }
                        catch (Exception ex)
                        {
                            Err = ex.Message.ToString();
                            UpdateStatus.AppendText(Err);
                        }

                    }
                    else
                    {
                        string Err = null;
                        try
                        {
                            agDrvr1.Acquisition.Hints.Add(AgM9392HintTypeEnum.AgM9392HintTypeStreamingMaxBufferSizeInBytes, 100000000);
                            agDrvr2.Acquisition.Hints.Add(AgM9392HintTypeEnum.AgM9392HintTypeStreamingMaxBufferSizeInBytes, 100000000);
                        }
                        catch (Exception ex)
                        {
                            Err = ex.Message.ToString();
                            UpdateStatus.AppendText(Err);
                        }
                    }



                    UpdateStatus.AppendText("Initiate Measurement on Slave!\n");
                    agDrvr2.Measurements.Initiate(); // required for streaming
                    DisplayErrors();
                    UpdateStatus.AppendText("Start Streaming on Slave!\n");
                    agDrvr2.Streaming.Start(); // required for streaming Start Slave Digitizer First
                    UpdateStatus.AppendText("Initiate Measurement on Master!\n");//callback

                    agDrvr1.Measurements.Initiate(); // required for streaming Initialize Master Digitizer First
                    DisplayErrors();
                    UpdateStatus.AppendText("Start Streaming on Master!\n");
                    agDrvr1.Streaming.Start(); // required for streaming
                    ReadBufferSizes();
                    DisplayErrors();




                    while (bWaitForTrigger)
                    {
                        UpdateStatus.AppendText("Waiting For Trigger 1...\n");//callback
                        CheckForTrigger();
                    }

                    this.Timer1.Enabled = true; // not used

                    string Error = null;

                    while (agDrvr1.Streaming.IsActive && !exit)
                    {
                        try
                        {
                            StreamingStatusUpdate();
                        }
                        catch (Exception err)
                        {
                            Error = err.Message.ToString();
                            UpdateStatus.AppendText(Error);
                        }
                        finally
                        {
                                                      
                        }

                        System.Threading.Thread.Sleep(1000);
                    }
                    UpdateStatus.AppendText("Recording Complete\n");

                }
            }
            this.Timer1.Enabled = false;

            progressBar1.Value = 0;
            
            if (debug)
            {
                agDrvr1.Status.ErrorVerbosity = AgM9392ErrorVerbosityEnum.AgM9392ErrorVerbosityDefault;
            }
        }





        private double CalculateBandWidthFromSampleRate()
        {
            double dSampleRate = 1;
            double dBandWidth = dSampleRate / 1.25;
            return dBandWidth;
        }



        private void Init_HW()
        {
            UpdateStatus.AppendText("Initializing Hardware..........\n");
            UpdateStatus.AppendText("..");
            Thread.Sleep(100);
            UpdateStatus.AppendText("....");
            Thread.Sleep(100);
            UpdateStatus.AppendText("......\n");
            UpdateStatus.AppendText("Please Wait......\n");
            Initialize_VSAs();
        }



        private void SetupTriggerMode()
        {
            UpdateStatus.AppendText("Setting Normal Trigger Mode!\n");

            if (triggerModeSet == Convert.ToInt16(triggerMode.Ext))  // External
            {
                if (NoOfChannels == 1)
                {
                    UpdateStatus.AppendText("Master Ext Trigger Set \n");
                    agDrvr1.Trigger.Source = Agilent.AgM9392.Interop.AgM9392TriggerSourceEnum.AgM9392TriggerSourceExternal;
                    agDrvr1.Trigger.External.Slope = AgM9392ExternalTriggerSlopeEnum.AgM9392ExternalTriggerSlopePositive;
                    agDrvr1.Trigger.External.Level = TriggerLevel;
                }
                else if (NoOfChannels == 2)
                {
                    UpdateStatus.AppendText("Master / Slave Ext Trigger Set\n");
                    agDrvr1.Trigger.Source = Agilent.AgM9392.Interop.AgM9392TriggerSourceEnum.AgM9392TriggerSourceExternal;
                    agDrvr1.Trigger.External.Slope = AgM9392ExternalTriggerSlopeEnum.AgM9392ExternalTriggerSlopePositive;
                    agDrvr1.Trigger.External.Level = TriggerLevel;
                    agDrvr2.Trigger.Source = Agilent.AgM9392.Interop.AgM9392TriggerSourceEnum.AgM9392TriggerSourceExternal;
                    agDrvr2.Trigger.External.Slope = AgM9392ExternalTriggerSlopeEnum.AgM9392ExternalTriggerSlopePositive;
                    agDrvr2.Trigger.External.Level = TriggerLevel;
                }

                bWaitForTrigger = true;

            }
            else if (triggerModeSet == Convert.ToInt16(triggerMode.Vid))  // Video trigger
            {
                if (NoOfChannels == 1)
                {
                    UpdateStatus.AppendText("Master Vid Trigger Set \n");
                    agDrvr1.Trigger.Source = Agilent.AgM9392.Interop.AgM9392TriggerSourceEnum.AgM9392TriggerSourceVideo;
                    agDrvr1.Trigger.Video.Slope = AgM9392VideoTriggerSlopeEnum.AgM9392VideoTriggerSlopePositive;
                    agDrvr1.Trigger.Video.Level = TriggerLevel;
                }

                else if (NoOfChannels == 2)
                {
                    UpdateStatus.AppendText("Master / Slave Vid Trigger Set \n");
                    agDrvr1.Trigger.Source = Agilent.AgM9392.Interop.AgM9392TriggerSourceEnum.AgM9392TriggerSourceVideo;
                    agDrvr1.Trigger.Video.Slope = AgM9392VideoTriggerSlopeEnum.AgM9392VideoTriggerSlopePositive;
                    agDrvr1.Trigger.Video.Level = TriggerLevel;
                    agDrvr2.Trigger.Source = Agilent.AgM9392.Interop.AgM9392TriggerSourceEnum.AgM9392TriggerSourceVideo;
                    agDrvr2.Trigger.Video.Slope = AgM9392VideoTriggerSlopeEnum.AgM9392VideoTriggerSlopePositive;
                    agDrvr2.Trigger.Video.Level = TriggerLevel;
                }


                bWaitForTrigger = true;
            }

            else if (triggerModeSet == Convert.ToInt16(triggerMode.Imm))  // Immediate
            {
                if (NoOfChannels == 1)
                {
                    UpdateStatus.AppendText("Master Immediate Trigger Set \n");
                    agDrvr1.Trigger.Source = Agilent.AgM9392.Interop.AgM9392TriggerSourceEnum.AgM9392TriggerSourceImmediate;
                }
                else if (NoOfChannels == 2)
                {
                    agDrvr1.Trigger.Source = Agilent.AgM9392.Interop.AgM9392TriggerSourceEnum.AgM9392TriggerSourceImmediate;
                    agDrvr2.Trigger.Source = Agilent.AgM9392.Interop.AgM9392TriggerSourceEnum.AgM9392TriggerSourceImmediate;
                    UpdateStatus.AppendText("Master / Slave Immediate Trigger is an unsupported Mode.\n");
                }

                bWaitForTrigger = false;
            }

            else if (triggerModeSet == Convert.ToInt16(triggerMode.Soft))  // Software -- Same as External - Different Level
            {
                if (NoOfChannels == 1)
                {
                    UpdateStatus.AppendText("Master Software Trigger Set - Uses Ext Trigger \n");
                    agDrvr1.Trigger.Source = Agilent.AgM9392.Interop.AgM9392TriggerSourceEnum.AgM9392TriggerSourceExternal;
                    agDrvr1.Trigger.External.Slope = AgM9392ExternalTriggerSlopeEnum.AgM9392ExternalTriggerSlopePositive;
                    agDrvr1.Trigger.External.Level = 1;
                }
                else if (NoOfChannels == 2)
                {
                    UpdateStatus.AppendText("Master / Slave Software Trigger Set - uses Master / Slave Trigger \n");
                    agDrvr1.Trigger.Source = Agilent.AgM9392.Interop.AgM9392TriggerSourceEnum.AgM9392TriggerSourceExternal;
                    agDrvr1.Trigger.External.Slope = AgM9392ExternalTriggerSlopeEnum.AgM9392ExternalTriggerSlopePositive;
                    agDrvr1.Trigger.External.Level = 1;
                    agDrvr2.Trigger.Source = Agilent.AgM9392.Interop.AgM9392TriggerSourceEnum.AgM9392TriggerSourceExternal;
                    agDrvr2.Trigger.External.Slope = AgM9392ExternalTriggerSlopeEnum.AgM9392ExternalTriggerSlopePositive;
                    agDrvr2.Trigger.External.Level = TriggerLevel;
                }

                bWaitForTrigger = true;
            }

            if (NoOfChannels == 1)
            {
                agDrvr1.Trigger.Delay = triggerDelay;
            }
            else if (NoOfChannels == 2)
            {
                agDrvr1.Trigger.Delay = triggerDelay;
                agDrvr2.Trigger.Delay = triggerDelay;
            }




        }

        private void SetupMultichannelTriggerMode()
        {
            SetupTriggerMode();
            UpdateStatus.AppendText("Multichannel Trigger Mode!\n");

            if (triggerModeSet == Convert.ToInt16(triggerMode.Ext))  // External
            {
                agDrvr1.Trigger.Source = AgM9392TriggerSourceEnum.AgM9392TriggerSourceExternal;
                agDrvr1.Trigger.Source = AgM9392TriggerSourceEnum.AgM9392TriggerSourceExternalSyncOut;//REQUIRED
                agDrvr1.Trigger.External.Level = TriggerLevel;
                agDrvr1.Trigger.External.Slope = AgM9392ExternalTriggerSlopeEnum.AgM9392ExternalTriggerSlopePositive;//REQUIRED

                if (NoOfChannels == 2)
                {
                    agDrvr2.Trigger.Source = AgM9392TriggerSourceEnum.AgM9392TriggerSourceExternal;//REQUIRED
                    agDrvr2.Trigger.External.Level = SlaveTriggerLevel;//REQUIRED
                    agDrvr2.Trigger.External.Slope = AgM9392ExternalTriggerSlopeEnum.AgM9392ExternalTriggerSlopePositive;//REQUIRED
                }

                bWaitForTrigger = true;
            }

            else if (triggerModeSet == Convert.ToInt16(triggerMode.Soft))  // Software -- Same as External - Different Level
            {
                agDrvr1.Trigger.Source = AgM9392TriggerSourceEnum.AgM9392TriggerSourceExternal;
                agDrvr1.Trigger.Source = AgM9392TriggerSourceEnum.AgM9392TriggerSourceExternalSyncOut;//REQUIRED
                agDrvr1.Trigger.External.Level = 1;
                agDrvr1.Trigger.External.Slope = AgM9392ExternalTriggerSlopeEnum.AgM9392ExternalTriggerSlopePositive;//REQUIRED

                if (NoOfChannels == 2)
                {
                    agDrvr2.Trigger.Source = AgM9392TriggerSourceEnum.AgM9392TriggerSourceExternal;//REQUIRED
                    agDrvr2.Trigger.External.Level = SlaveTriggerLevel;//REQUIRED
                    agDrvr2.Trigger.External.Slope = AgM9392ExternalTriggerSlopeEnum.AgM9392ExternalTriggerSlopePositive;//REQUIRED
                }

                bWaitForTrigger = true;
            }

            else if (triggerModeSet == Convert.ToInt16(triggerMode.Vid))  // Vid
            {
                agDrvr1.Trigger.Source = AgM9392TriggerSourceEnum.AgM9392TriggerSourceVideo;
                agDrvr1.Trigger.Source = AgM9392TriggerSourceEnum.AgM9392TriggerSourceVideoSyncOut;//REQUIRED
                agDrvr1.Trigger.External.Level = SlaveTriggerLevel;//REQUIRED
                agDrvr1.Trigger.External.Slope = AgM9392ExternalTriggerSlopeEnum.AgM9392ExternalTriggerSlopePositive;//REQUIRED
                agDrvr1.Trigger.Video.Level = TriggerLevel;//REQUIRED
                agDrvr1.Trigger.Video.Slope = AgM9392VideoTriggerSlopeEnum.AgM9392VideoTriggerSlopePositive;//REQUIRED

                if (NoOfChannels == 2)
                {
                    agDrvr2.Trigger.Source = AgM9392TriggerSourceEnum.AgM9392TriggerSourceExternal;//REQUIRED
                    agDrvr2.Trigger.External.Level = SlaveTriggerLevel;//REQUIRED
                    agDrvr2.Trigger.External.Slope = AgM9392ExternalTriggerSlopeEnum.AgM9392ExternalTriggerSlopePositive;//REQUIRED
                }

                bWaitForTrigger = true;
            }
            else
            {
                if (UsingStreamingMode)
                {
                    UpdateStatus.AppendText("Immediate Trigger Mode Not Supported For Multichannel Trigger!\n");
                }
            }

        }

        private void checkBox1_Simulate_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox1_Simulate.Checked)
               Options = "Simulate=true";
            else
               Options = "Simulate=false";

               Change_HW_Configuration();

        }

        private void GetSimulateState()
        {

            if (checkBox1_Simulate.Checked)
                Options = "Simulate=true";
            else
                Options = "Simulate=false";

            IsSimulate = checkBox1_Simulate.Checked;
        }



        private void StreamingStatusUpdate()
        {
            

            if (debug)
            {
                double dSampleRate = agDrvr1.Measurements.SampleRate;
                double dFreq = agDrvr1.Acquisition.Frequency;
                double dBW = agDrvr1.Acquisition.Bandwidth;
                double dAcqTime = agDrvr1.Acquisition.Time;
                long iSamplesRequested = agDrvr1.Streaming.SamplesRequested;
                long iSamplesAcquired = agDrvr1.Streaming.SamplesAcquired;
                long iSamplesWritten = agDrvr1.Streaming.SamplesWritten;
            }
            int NoOfErrors = agDrvr1.Status.NumberOfErrors;
            if (NoOfErrors > 0)
            {
                int errCode = 0;
                string ErrorMessage = null;
                agDrvr1.Utility.ErrorQuery(ref errCode, ref ErrorMessage);
                string ErrorLine = null;
                ErrorLine = "\nThere Was an Error! ErrorCode:" + errCode + "  Error Message: " + ErrorMessage + "\n\n";
                Thread.Sleep(100);
                UpdateStatus.AppendText(ErrorLine);
            }

            string sStatus = "Acquired " + agDrvr1.Streaming.SamplesAcquired.ToString() + " Samples, " +
                             "Wrote " + agDrvr1.Streaming.SamplesWritten.ToString() + " Samples, " +
                             "Recording " + ((double)(agDrvr1.Streaming.SamplesWritten) / (agDrvr1.Streaming.SamplesRequested) * 100).ToString() + "% Complete, " +
                             "Acquisition OverLoaded? " + agDrvr1.Measurements.DidAcquisitionOverload.ToString() + "\n";

            ProgressCurrentPercent = (int)((float)(agDrvr1.Streaming.SamplesWritten) / (agDrvr1.Streaming.SamplesRequested) * 100);
            if (ProgressCurrentPercent > 100)
                ProgressCurrentPercent = 100;

            progressBar1.Value = ProgressCurrentPercent;
            UpdateStatus.AppendText(sStatus); ;
        }

        private void numericUpDown3AcquisitiontimeSec_ValueChanged(object sender, EventArgs e)
        {
            acquisitionTime = Convert.ToDouble(numericUpDown3AcquisitiontimeSec.Value);
            agDrvr1.Acquisition.PreviewCharacteristics(frequency, bandwidth, acquisitionTime, ref actualBW, ref dSampleRate, ref NoSamples);
            ResourceManagement();
        }

        private void GetAcquisitionTime()
        {
            acquisitionTime = Convert.ToDouble(numericUpDown3AcquisitiontimeSec.Value);
        }

        private void numericUpDown1_ValueChanged(object sender, EventArgs e)
        {
            power = Convert.ToDouble(numericUpDown1.Value);
        }

        private void GetExpectedPowerdBm()
        {
            power = Convert.ToDouble(numericUpDown1.Value);
        }

        private void button1Start_Click(object sender, EventArgs e)
        {
            if (panel2.BackColor == Color.Green && panel3.BackColor == Color.Green && panel4.BackColor == Color.Green)
            {
                this.progressBar1.Value = 0;
                this.Timer1.Interval = 1000;// 1 second



                if (UsingStreamingMode)
                {
                    UpdateStatus.AppendText("In Streaming Mode...\n");
                    //Start Streaming
                    if (NoOfChannels == 1)
                        SingleChannelStreamingSetup();
                    else if (NoOfChannels == 2)
                        DualChannelStreamingSetup();

                }
            }
            else
            {
                if (panel2.BackColor == Color.Red || panel3.BackColor == Color.Red)
                {
                    UpdateStatus.AppendText("NOT ENOUGH DISK SPACE FOR RECORDING!\n");
                }
                if (panel4.BackColor == Color.Red)
                {
                    UpdateStatus.AppendText("NOT ENOUGH FREE RAM FOR RECORDING!\n");
                }
                else
                    UpdateStatus.AppendText("UNKNOWN ERROR....CHECK SETTINGS!\n");
            }
        }

        



        private void numericUpDown2TriggerLevel_ValueChanged(object sender, EventArgs e)
        {
            TriggerLevel = Convert.ToDouble(numericUpDown2TriggerLevel.Value);
        }

        private void GetTriggerLevel()
        {
            TriggerLevel = Convert.ToDouble(numericUpDown2TriggerLevel.Value);
        }

        private void numericUpDown2TrigDelay_ValueChanged(object sender, EventArgs e)
        {
            triggerDelay = Convert.ToDouble(numericUpDown2TrigDelay.Value);
            if (triggerDelay != 0)
            {
                UpdateStatus.AppendText("Pre-Trigger Time is included in Acquisition Time!\n");
            }
            ResourceManagement();
        }

        private void GetTriggerDelay()
        {
            triggerDelay = Convert.ToDouble(numericUpDown2TrigDelay.Value);
        }

        private void comboBox1HWConfig_SelectedIndexChanged(object sender, EventArgs e)
        {

        }

        private void checkedListBox3_HW_Config_SelectedIndexChanged(object sender, EventArgs e)
        {
            
                    GetPreselector();
                    if (checkedListBox3_HW_Config.SelectedIndex == 0)
                    {
                        checkedListBox3_HW_Config.SetItemChecked(1, false);//toggle state
                        checkedListBox3_HW_Config.SetItemChecked(0, true);//toggle state
                        FullVSAConfig = true;
                        UpdateStatus.AppendText("VSA Channels Selected with Independant LO's!\n");
                        if (Is_WideEnough()!=true)
                        {
                            UpdateStatus.AppendText("VSA Channels do not support more than 40MHz Bandwidth at this Center Frequency!\n");
                            UpdateStatus.AppendText("Setting Bandwidth to 40MHz.\n");
                            numericUpDown2BandwidthMHz.Value = 40;
                        }
                    }
                    if (checkedListBox3_HW_Config.SelectedIndex == 1)
                    {
                        checkedListBox3_HW_Config.SetItemChecked(0, false);//toggle state
                        checkedListBox3_HW_Config.SetItemChecked(1, true);//toggle state

                        FullVSAConfig = false;
                        numericUpDown1FrequencyMHz.Value = 500;
                        numericUpDown1.Value = -4;
                        UpdateStatus.AppendText("Setting Frequency to 500MHz \nSetting Expected Power Level -4dBm \n\n");
                        UpdateStatus.AppendText("VSA Channels Selected with shared LO!\n");
                    }
                    if (NoOfChannels == 2)
                    {
                        CheckRecordingMode();
                    }
                    ResourceManagement();
                 
        }

        private void Get_HW_Config()
        {
            if (checkedListBox3_HW_Config.SelectedIndex == 0)
            {
                checkedListBox3_HW_Config.SetItemChecked(0, true);//toggle state
                checkedListBox3_HW_Config.SetItemChecked(1, false);//toggle state
                FullVSAConfig = true;
            }
            if (checkedListBox3_HW_Config.SelectedIndex == 1)
            {
                checkedListBox3_HW_Config.SetItemChecked(1, true);
                checkedListBox3_HW_Config.SetItemChecked(0, false);//toggle state
                FullVSAConfig = false;
            }
        }

        private void Set_HW_Config()
        {
            if (FullVSAConfig)
            {
                if (checkedListBox3_HW_Config.SelectedIndex == 1)
                {
                    checkedListBox3_HW_Config.SetItemChecked(0, true);//toggle state
                }
            }
            else
            {
                if (checkedListBox3_HW_Config.SelectedIndex == 0)
                {
                    checkedListBox3_HW_Config.SetItemChecked(1, true);
                }
            }
        }

        public void SetStorage(string text)
        {
        }

        private void textBox1CH1_Storage_TextChanged(object sender, EventArgs e)
        {

        }

        private void textBox2CH2_Storage_TextChanged(object sender, EventArgs e)
        {

        }

        private void textBox3FreeRAM_TextChanged(object sender, EventArgs e)
        {

        }

        private void textBox1RAMRequired_TextChanged(object sender, EventArgs e)
        {
            string RAM = Convert.ToString(TotalRAMReqMB);
            textBox1RAMRequired.Text = RAM;
        }

        private void UpdateTotalRAMRequired()
        {
            string ram = Convert.ToString(TotalRAMReqMB);
            textBox1RAMRequired.Text = ram;
        }

        private void CheckForTrigger()
        {
            string Error = null;

            if (agDrvr1.Measurements.WaitForTrigger(timeOut) == AgM9392WaitStatusEnum.AgM9392WaitStatusComplete)
            {
                UpdateStatus.AppendText("Trigger Received!\n");
                bWaitForTrigger = false;

            }
            else if (agDrvr1.Measurements.WaitForTrigger(timeOut) == AgM9392WaitStatusEnum.AgM9392WaitStatusWaitExpired)
            {
                UpdateStatus.AppendText("Trigger Timed Out!\n");
                bWaitForTrigger = false;

                try
                {


                    if (NoOfChannels == 1)
                    {
                        UpdateStatus.AppendText("Aborting Channel 1 Measurement!\n");
                        agDrvr1.Measurements.Abort();// Abort streaming acquisition on Channel 1

                    }
                    else if (NoOfChannels == 2)
                    {
                        UpdateStatus.AppendText("Aborting Channel 1 Measurement!\n");
                        agDrvr1.Measurements.Abort();// Abort streaming acquisition on Channel 1
                        UpdateStatus.AppendText("Aborting Channel 2 Measurement!\n");
                        agDrvr2.Measurements.Abort();// Abort streaming acquisition on Channel 2
                    }

                }
                catch (Exception err)
                {
                    Error = err.Message.ToString();
                    UpdateStatus.AppendText(Error);
                }
            }
        }

        private void UpdateGlobalVariables()
        {
            Thread.Sleep(1000);
            GetAcquisitionTime();
            Get_IVIResources();
            GetSimulateState();
            GetAcquisitionTime();
            GetBandWidth();
            GetDigitizerOnlyState();
            GetStreamingFiles();
            GetDriveVolumeMaster();
            GetDriveVolumeSlave1();
            SetRAMStorageLEDWarning();
            GetExpectedPowerdBm();

            GetFrequency();
            GetTimeOut();
            NoOfChannels = Set_No_Of_Channels();

            GetNoOfChannels();

            GetRecordingMode();
            GetSerialPortTrigger();
            GetStorageConfig();
            GetTriggerDelay();
            GetTriggerLevel();
            GetTriggerMode();
            GetTriggerSlope();





        }

        private void ResourceManagement()
        {
            GetNoOfChannels();
            Get_HW_Config();
            GetRecordingMode();
            GetStorageConfig();
            CheckTriggerMode();
            dDiskSpaceRequired = CalculateDiskSpaceRequirement(acquisitionTime, dSampleRate, triggerDelay);
            TotalRAMReqMB = Calculate_Required_RAM(dSampleRate, acquisitionTime, triggerDelay, NoOfChannels);
            UpdateTotalRAMRequired();
            SetChan1StorageLEDWarning();
            SetChan2StorageLEDWarning();
            SetRAMStorageLEDWarning();

        }

        private void label16_Click(object sender, EventArgs e)
        {

        }



        private void SetChan1StorageLEDWarning()
        {
            if (NoOfChannels == 1)
            {
                if (dDiskSpaceRequired > (CH1DiskSpaceGB * 1e9))
                {
                    panel2.BackColor = Color.Red;//fail
                }
                else
                {
                    panel2.BackColor = Color.Green;//pass
                    panel3.BackColor = Color.Green;//pass
                }
            }

            else if (NoOfChannels == 2)
            {
                if (2 * dDiskSpaceRequired > (CH2DiskSpaceGB * 1e9))
                {
                    if (CheckVolume())
                    {
                        panel2.BackColor = Color.Red;//fail
                        UpdateStatus.AppendText("Not Enough Disk Space for Acquisition!\n");

                    }
                    else
                        panel2.BackColor = Color.Green;//pass

                }
                else
                {
                    if (dDiskSpaceRequired > (CH2DiskSpaceGB * 1e9))
                    {
                        panel3.BackColor = Color.Red;//fail
                        UpdateStatus.AppendText("Not Enough Disk Space for Acquisition!\n");//callback
                    }
                    else
                    {
                        panel3.BackColor = Color.Green;//pass
                        panel2.BackColor = Color.Green;//pass

                    }
                }


            }

        }


        private void SetChan2StorageLEDWarning()
        {
            if (NoOfChannels == 2)
            {
                if (CheckVolume())
                {
                    if (2 * dDiskSpaceRequired > (CH2DiskSpaceGB * 1e9))
                    {
                        panel2.BackColor = Color.Red;//fail
                        panel3.BackColor = Color.Red;//fail
                        UpdateStatus.AppendText("Not Enough Disk Space for Acquisition!\n");
                    }
                    else
                    {
                        panel3.BackColor = Color.Green;//pass
                    }
                }
                else
                {
                    if (dDiskSpaceRequired > (CH2DiskSpaceGB * 1e9))
                    {

                        if (CheckVolume())
                            panel2.BackColor = Color.Red;//fail

                        UpdateStatus.AppendText("Not Enough Disk Space for Acquisition!\n");
                    }
                    else
                        panel3.BackColor = Color.Green;//pass

                }


            }
        }

        private void SetRAMStorageLEDWarning()
        {
            if (NoOfChannels == 1)
            {
                if (TotalRAMReqMB > (FreeRAM_MB))
                {
                    panel4.BackColor = Color.Red;//fail
                    UpdateStatus.AppendText("Not Enough Free RAM for Acquisition!\n");
                }
                else
                    panel4.BackColor = Color.Green;//pass

            }

            else if (NoOfChannels == 2)
            {
                if (TotalRAMReqMB > (FreeRAM_MB))
                {
                    panel4.BackColor = Color.Red;//fail
                    UpdateStatus.AppendText("Not Enough Free RAM for Acquisition!\n");
                }
                else
                    panel4.BackColor = Color.Green;//pass
            }


        }

       
        private void DisplayErrors()
        {
            Thread.Sleep(10);
            if (true)
            {
                int errorCode = 0;
                string ErrorMessage = null;
                string WriteError = null;
                // read paramater errors!

                agDrvr1.Utility.ErrorQuery(ref errorCode, ref ErrorMessage);
                if (errorCode != 0)
                {
                    WriteError = "MasterVSA ErrorCode: " + errorCode + "   ErrorMessage: " + ErrorMessage + "\n";
                    UpdateStatus.AppendText(WriteError);
                }
                agDrvr2.Utility.ErrorQuery(ref errorCode, ref ErrorMessage);
                if (errorCode != 0)
                {
                    WriteError = "SlaveVSA ErrorCode: " + errorCode + "   ErrorMessage: " + ErrorMessage + "\n";
                    UpdateStatus.AppendText(WriteError);
                }
            }
        }

        private void progressBar1_Click(object sender, EventArgs e)
        {

        }




        protected override void OnResize(EventArgs e)
        {
            // invalidate the control to get a repaint
            this.Invalidate();
        }

        protected override void OnPaint(PaintEventArgs e)
        {
            g = e.Graphics;
            brush = new SolidBrush(BarColor);


            Rectangle rect = this.ClientRectangle;

            // Calculate area for drawing the progress
            rect.Width = (int)((float)rect.Width * ProgressCurrentPercent);

            // draw progress meter
            g.FillRectangle(brush, rect);



        }






        private void label14_Click(object sender, EventArgs e)
        {

        }

        private void label9_Click(object sender, EventArgs e)
        {

        }

        private void panel2_Paint(object sender, PaintEventArgs e)
        {


        }

        private void panel3_Paint(object sender, PaintEventArgs e)
        {

        }

        private void textBox1StreamName1_MouseClick(object sender, MouseEventArgs e)
        {

        }

        private void textBox2StreamName2_MouseClick(object sender, MouseEventArgs e)
        {

        }


        private void EndWaitForTrigger()
        {

          
        }





        private void button1_SelectMasterFile_Click(object sender, EventArgs e)
        {
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.Filter = "Streaming File|*.bin";
            saveFileDialog1.Title = "Save a Binary File";
            saveFileDialog1.ShowDialog();


            if (saveFileDialog1.FileName != "")
            {

                System.IO.FileStream fs =
                   (System.IO.FileStream)saveFileDialog1.OpenFile();
                textBox1StreamName1.Text = saveFileDialog1.FileName.ToString();

                fs.Close();
            }
        }

        private void button2_SlaveFile_Click(object sender, EventArgs e)
        {
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.Filter = "Streaming File|*.bin";
            saveFileDialog1.Title = "Save a Binary File";
            saveFileDialog1.ShowDialog();


            if (saveFileDialog1.FileName != "")
            {

                System.IO.FileStream fs =
                   (System.IO.FileStream)saveFileDialog1.OpenFile();
                textBox2StreamName2.Text = saveFileDialog1.FileName.ToString();

                fs.Close();
            }
        }


        private void SaveConfigFile()
        {
            SaveFileDialog saveFileDialog1 = new SaveFileDialog();
            saveFileDialog1.Filter = "Streaming Config|*.txt";
            saveFileDialog1.Title = "Save a Config File";
            saveFileDialog1.ShowDialog();


            if (saveFileDialog1.FileName != "")
            {
                StreamWriter sw = new StreamWriter(saveFileDialog1.OpenFile());

                foreach (String Item in ConfigData)
                {
                    sw.WriteLine(Item);
                }

                sw.Close();
            }
        }


        private void LoadConfigFile()
        {
            string Item = null;
            ConfigData.Clear();

            OpenFileDialog openFileDialog1 = new OpenFileDialog();
            openFileDialog1.Filter = "Streaming Config|*.txt";
            openFileDialog1.Title = "Load a Config File";
            openFileDialog1.ShowDialog();


            if (openFileDialog1.FileName != "")
            {

                StreamReader sr = new StreamReader(openFileDialog1.OpenFile());

                while (!sr.EndOfStream)
                {
                    Item = sr.ReadLine();
                    ConfigData.Add(Item);
                }

                sr.Close();
            }
        }






        private void GetCurrentConfigStates()
        {

            List<KeyValuePair<string, string>> dataEntries = dict.ToList();

            foreach (KeyValuePair<string, string> dataEntry in dataEntries)
            {
                if (dataEntry.Key == "VSACONFIG")
                {
                    dataEntry.Value.Replace(dataEntry.Value, FullVSAConfig.ToString());
                }
                if (dataEntry.Key == "SIMULATE")
                {
                    dataEntry.Value.Replace(dataEntry.Value, IsSimulate.ToString());
                }
                if (dataEntry.Key == "MASTERRESOURCE")
                {
                    dataEntry.Value.Replace(dataEntry.Value, resourceDesc1.ToString());
                }
                if (dataEntry.Key == "SLAVERESOURCE")
                {
                    dataEntry.Value.Replace(dataEntry.Value, resourceDesc2.ToString());
                }
                if (dataEntry.Key == "RAID")
                {
                    dataEntry.Value.Replace(dataEntry.Value, UsesRAID.ToString());
                }
                if (dataEntry.Key == "MASTERSTREAMINGFILE")
                {
                    dataEntry.Value.Replace(dataEntry.Value, streamFile1.Trim().ToString());
                }
                if (dataEntry.Key == "SLAVESTREAMINGFILE")
                {
                    dataEntry.Value.Replace(dataEntry.Value, streamFile2.Trim().ToString());
                }
                if (dataEntry.Key == "TRIGGERMODE")
                {
                    dataEntry.Value.Replace(dataEntry.Value, triggerModeSet.ToString());
                }
                if (dataEntry.Key == "SOFTWARETRIGGERSET")
                {
                    dataEntry.Value.Replace(dataEntry.Value, SoftwareTrigger.ToString());
                }
                if (dataEntry.Key == "TRIGGERLEVEL")
                {
                    dataEntry.Value.Replace(dataEntry.Value, TriggerLevel.ToString());
                }
                if (dataEntry.Key == "SLAVETRIGGERLEVEL")
                {
                    dataEntry.Value.Replace(dataEntry.Value, SlaveTriggerLevel.ToString());
                }
                if (dataEntry.Key == "TRIGGERDELAY")
                {
                    dataEntry.Value.Replace(dataEntry.Value, triggerDelay.ToString());
                }
                if (dataEntry.Key == "TRIGGERTIMEOUT")
                {
                    dataEntry.Value.Replace(dataEntry.Value, timeOut.ToString());
                }
                if (dataEntry.Key == "TRIGGERSLOPE")
                {
                    dataEntry.Value.Replace(dataEntry.Value, trigSlopeSet.ToString());
                }
                if (dataEntry.Key == "FREQUENCY")
                {
                    dataEntry.Value.Replace(dataEntry.Value, frequency.ToString());
                }
                if (dataEntry.Key == "NUMCHANNELS")
                {
                    dataEntry.Value.Replace(dataEntry.Value, NoOfChannels.ToString());
                }
                if (dataEntry.Key == "BANDWIDTH")
                {
                    dataEntry.Value.Replace(dataEntry.Value, bandwidth.ToString());
                }
                if (dataEntry.Key == "AQUISITIONTIME")
                {
                    dataEntry.Value.Replace(dataEntry.Value, acquisitionTime.ToString());
                }
                if (dataEntry.Key == "EXPECTEDPOWER")
                {
                    dataEntry.Value.Replace(dataEntry.Value, power.ToString());
                }
                if (dataEntry.Key == "USEPRESELECTOR")
                {
                    dataEntry.Value.Replace(dataEntry.Value, UsePreselector.ToString());
                }
                if (dataEntry.Key == "DIGITIZERONLYMODE")
                {
                    dataEntry.Value.Replace(dataEntry.Value, UsesDigOnlyMode.ToString());
                }
                if (dataEntry.Key == "RECORDMODE")
                {
                    dataEntry.Value.Replace(dataEntry.Value, recordMode.ToString());
                }
            }
            dict.Clear();
            InitializeConfigStates();


            CreateConfigData();

        }

        private void InitializeConfigStates()
        {

            dict.Add("VSACONFIG", FullVSAConfig.ToString());
            dict.Add("SIMULATE", IsSimulate.ToString());
            dict.Add("MASTERRESOURCE", resourceDesc1.Trim().ToString());
            dict.Add("SLAVERESOURCE", resourceDesc2.Trim().ToString());
            dict.Add("NUMCHANNELS", NoOfChannels.ToString());
            dict.Add("RAID", UsesRAID.ToString());
            dict.Add("MASTERSTREAMINGFILE", streamFile1.Trim().ToString());
            dict.Add("SLAVESTREAMINGFILE", streamFile2.Trim().ToString());
            dict.Add("TRIGGERMODE", triggerModeSet.ToString());
            dict.Add("SOFTWARETRIGGERSET", SoftwareTrigger.ToString());
            dict.Add("TRIGGERLEVEL", TriggerLevel.ToString());
            dict.Add("SLAVETRIGGERLEVEL", SlaveTriggerLevel.ToString());
            dict.Add("TRIGGERDELAY", triggerDelay.ToString());
            dict.Add("TRIGGERTIMEOUT", timeOut.ToString());
            dict.Add("TRIGGERSLOPE", trigSlopeSet.ToString());
            dict.Add("FREQUENCY", frequency.ToString());
            dict.Add("BANDWIDTH", bandwidth.ToString());
            dict.Add("AQUISITIONTIME", acquisitionTime.ToString());
            dict.Add("EXPECTEDPOWER", power.ToString());
            dict.Add("USEPRESELECTOR", UsePreselector.ToString());
            dict.Add("DIGITIZERONLYMODE", UsesDigOnlyMode.ToString());
            dict.Add("RECORDMODE", recordMode.ToString());

            CreateConfigData();

        }

        private void UpdateConfigStates()
        {
            LineItem = null;


            List<KeyValuePair<string, string>> dataEntries = dict.ToList();

            foreach (KeyValuePair<string, string> dataEntry in dataEntries)
            {
                if (dataEntry.Key == "VSACONFIG")
                {
                    FullVSAConfig = Convert.ToBoolean(dataEntry.Value);

                    int checkeditem = 0;
                    if (FullVSAConfig)
                    {
                        checkeditem = checkedListBox3_HW_Config.SelectedIndex;
                        checkedListBox3_HW_Config.SetSelected(0, true);
                        checkedListBox3_HW_Config.SetSelected(1, false);
                    }
                    else
                    {
                        checkedListBox3_HW_Config.SetSelected(0, false);
                        checkedListBox3_HW_Config.SetSelected(1, true);
                    }


                }
                if (dataEntry.Key == "SIMULATE")
                {
                    IsSimulate = Convert.ToBoolean(dataEntry.Value);
                    if (IsSimulate)
                    {
                        checkBox1_Simulate.CheckState = CheckState.Checked;
                        Options = "Simulate=true";
                    }
                    else
                    {
                        checkBox1_Simulate.CheckState = CheckState.Unchecked;
                        Options = "Simulate=false";
                    }
                    ResourceManagement();
                }
                if (dataEntry.Key == "MASTERRESOURCE")
                {
                    resourceDesc1 = dataEntry.Value.ToString();
                    Ch1_Resource.Text = resourceDesc1;

                }
                if (dataEntry.Key == "SLAVERESOURCE")
                {
                    resourceDesc2 = dataEntry.Value.ToString();
                    Ch2_Resource.Text = resourceDesc2;
                }
                if (dataEntry.Key == "RAID")
                {
                    UsesRAID = Convert.ToBoolean(dataEntry.Value);

                    if (UsesRAID)
                    {
                        checkedListBox2_Storage.SetItemChecked(0, true);//
                        checkedListBox2_Storage.SetItemChecked(1, false);//toggle
                    }
                    else
                    {
                        checkedListBox2_Storage.SetItemChecked(1, true);//
                        checkedListBox2_Storage.SetItemChecked(0, false);//toggle state
                    }

                }
                if (dataEntry.Key == "MASTERSTREAMINGFILE")
                {
                    streamFile1 = dataEntry.Value.ToString();
                    textBox1StreamName1.Text = streamFile1;

                }
                if (dataEntry.Key == "SLAVESTREAMINGFILE")
                {
                    streamFile2 = dataEntry.Value.ToString();
                    textBox2StreamName2.Text = streamFile2;
                }
                if (dataEntry.Key == "TRIGGERMODE")
                {
                    triggerModeSet = Convert.ToInt16(dataEntry.Value);
                    if (triggerModeSet == 0)
                    {
                        checkedListBox1_TriggerMode.SetItemChecked(0, true);//toggle state
                        checkedListBox1_TriggerMode.SetItemChecked(1, false);//toggle state
                        checkedListBox1_TriggerMode.SetItemChecked(2, false);//toggle state
                        numericUpDown2TriggerLevel.Value = 1; // default Ext trigger level

                    }
                    if (triggerModeSet == 1)
                    {
                        checkedListBox1_TriggerMode.SetItemChecked(0, false);//toggle state
                        checkedListBox1_TriggerMode.SetItemChecked(1, true);//toggle state
                        checkedListBox1_TriggerMode.SetItemChecked(2, false);//toggle state
                        numericUpDown2TriggerLevel.Value = -20; // default Vid trigger level
                    }


                    if (triggerModeSet == 2)
                    {
                        if (NoOfChannels == 2)
                        {
                            checkedListBox1_TriggerMode.SetItemChecked(1, true);// force Vid Trigger
                            checkedListBox1_TriggerMode.SetItemChecked(0, false);//toggle state
                            checkedListBox1_TriggerMode.SetItemChecked(2, false);//toggle state
                        }
                        else
                        {
                            checkedListBox1_TriggerMode.SetItemChecked(0, false);//toggle state
                            checkedListBox1_TriggerMode.SetItemChecked(1, false);//toggle state
                            checkedListBox1_TriggerMode.SetItemChecked(2, true);//toggle state
                        }
                    }




                }
                if (dataEntry.Key == "SOFTWARETRIGGERSET")
                {
                    SoftwareTrigger = Convert.ToBoolean(dataEntry.Value);
                    if (SoftwareTrigger)
                    {
                        radioButton2SerialPortTrigger.Checked = true;
                    }
                    else
                    {
                        radioButton2SerialPortTrigger.Checked = false;
                    }
                }
                if (dataEntry.Key == "TRIGGERLEVEL")
                {
                    TriggerLevel = Convert.ToDouble(dataEntry.Value);
                    numericUpDown2TriggerLevel.Value = (decimal)TriggerLevel;
                }
                if (dataEntry.Key == "SLAVETRIGGERLEVEL")
                {
                    SlaveTriggerLevel = Convert.ToDouble(dataEntry.Value);
                    numericUpDown3_SlaveTrigLev.Value = (decimal)SlaveTriggerLevel;
                }
                if (dataEntry.Key == "TRIGGERDELAY")
                {
                    triggerDelay = Convert.ToDouble(dataEntry.Value);
                    numericUpDown2TrigDelay.Value = (decimal)triggerDelay;
                }
                if (dataEntry.Key == "TRIGGERTIMEOUT")
                {
                    timeOut = Convert.ToInt16(dataEntry.Value);
                    numericUpDown2.Value = (decimal)timeOut;
                }
                if (dataEntry.Key == "TRIGGERSLOPE")
                {
                    trigSlopeSet = Convert.ToInt16(dataEntry.Value);
                    if (trigSlopeSet == 1)
                    {
                        checkedListBox6TriggerSlope.SetItemChecked(0, true);
                        checkedListBox6TriggerSlope.SetItemChecked(1, false);
                    }
                    else if (trigSlopeSet == 2)
                    {
                        checkedListBox6TriggerSlope.SetItemChecked(0, false);
                        checkedListBox6TriggerSlope.SetItemChecked(1, true);
                    }


                }
                if (dataEntry.Key == "FREQUENCY")
                {
                    frequency = Convert.ToDouble(dataEntry.Value);
                    numericUpDown1FrequencyMHz.Value = (decimal)(frequency / 1e6);
                }
                if (dataEntry.Key == "NUMCHANNELS")
                {
                    NoOfChannels = Convert.ToInt16(dataEntry.Value);
                    if (NoOfChannels == 1)
                    {
                        checkedListBox4_No_Of_Chans.SetItemChecked(0, true);//toggle state
                        checkedListBox4_No_Of_Chans.SetItemChecked(1, false);//toggle state

                    }
                    if (NoOfChannels == 2)
                    {
                        checkedListBox4_No_Of_Chans.SetItemChecked(0, false);//toggle state
                        checkedListBox4_No_Of_Chans.SetItemChecked(1, true);//toggle state
                    }
                    //ResourceManagement();
                }
                if (dataEntry.Key == "BANDWIDTH")
                {
                    bandwidth = Convert.ToDouble(dataEntry.Value);
                    numericUpDown2BandwidthMHz.Value = (decimal)(bandwidth / 1e6);
                }
                if (dataEntry.Key == "AQUISITIONTIME")
                {
                    acquisitionTime = Convert.ToDouble(dataEntry.Value);
                    numericUpDown3AcquisitiontimeSec.Value = (decimal)acquisitionTime;
                }
                if (dataEntry.Key == "EXPECTEDPOWER")
                {
                    power = Convert.ToDouble(dataEntry.Value);
                    numericUpDown1.Value = (decimal)power;

                }
                if (dataEntry.Key == "USEPRESELECTOR")
                {
                    UsePreselector = Convert.ToBoolean(dataEntry.Value);
                    if (UsePreselector)
                    {
                        checkBox1UsePreselector.Checked = true;
                    }
                    else
                        checkBox1UsePreselector.Checked = false;
                }
                if (dataEntry.Key == "DIGITIZERONLYMODE")
                {
                    UsesDigOnlyMode = Convert.ToBoolean(dataEntry.Value);
                    if (UsesDigOnlyMode)
                    {
                        radioButton3DigitizerOnlyMode.Checked = true;
                    }
                    else
                        radioButton3DigitizerOnlyMode.Checked = false;
                }
                if (dataEntry.Key == "RECORDMODE")
                {
                    recordMode = Convert.ToInt16(dataEntry.Value);

                    if (recordMode == 0)
                    {
                        checkedListBox1RecordMode.SetItemChecked(0, true);//toggle state
                        checkedListBox1RecordMode.SetItemChecked(1, false);//toggle state
                    }
                    else
                    {
                        checkedListBox1RecordMode.SetItemChecked(0, false);//toggle state
                        checkedListBox1RecordMode.SetItemChecked(1, true);//toggle state
                    }
                }


            }
            ResourceManagement();
            Change_HW_Configuration();
        }

        private void CreateConfigData()
        {
            ConfigData.Clear();
            string LineItem = null;

            List<KeyValuePair<string, string>> dataEntries = dict.ToList();
            foreach (KeyValuePair<string, string> dataEntry in dataEntries)
            {
                LineItem = string.Format("{0} {1}", dataEntry.Key.ToString(), dataEntry.Value);

                ConfigData.Add(LineItem);
            }

        }

        private void LoadConfigData()
        {
            LineItem = null;
            string[] ForParse = null;
            string Value = null;
            string Key = null;
            int maxIndex = 0;

            int i = 0;
            List<KeyValuePair<string, string>> dataEntries = dict.ToList();
            maxIndex = ConfigData.Count();
            dict.Clear();
            foreach (KeyValuePair<string, string> dataEntry in dataEntries)
            {

                foreach (string Item in ConfigData)
                {
                    while (i < maxIndex)
                    {
                        LineItem = (string)ConfigData[i].ToString();
                        ForParse = LineItem.Split(' ');
                        Key = ForParse[0].Trim().ToString();
                        Value = ForParse[1].Trim().ToString();
                        dict.Add(Key, Value);
                        i++;
                    }

                }


            }
        }

        private void numericUpDown2_ValueChanged_1(object sender, EventArgs e)
        {
            timeOut = (int)numericUpDown2.Value;
        }

        private void GetTimeOut()
        {
            timeOut = (int)numericUpDown2.Value;
        }

        private void saveFileDialog1_FileOk(object sender, CancelEventArgs e)
        {

        }

        private void saveConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            UpdateStatus.AppendText("Saving Current States to Configuration File!\n");
            GetCurrentConfigStates();
            CreateConfigData();
            SaveConfigFile();
            UpdateStatus.AppendText("Done!\n");
        }

        private void LoadConfigToolStripMenuItem_Click(object sender, EventArgs e)
        {
            UpdateStatus.AppendText("Loading States from Configuration File!\n");
            LoadConfigFile();
            LoadConfigData();
            UpdateConfigStates();
            UpdateStatus.AppendText("Done!\n");
        }

        private void checkBox1UsePreselector_CheckedChanged(object sender, EventArgs e)
        {
            if (checkBox1UsePreselector.Checked)
                UsePreselector = true;
            else
                UsePreselector = false;

            GetPreselector();
        }

        private void aboutToolStripMenuItem_Click(object sender, EventArgs e)
        {
            UpdateStatus.AppendText("\n Agilent Technologies Multichannel Streaming Example Program Version 1.0.0 \n\n");
        }

        private void fileToolStripMenuItem_Click(object sender, EventArgs e)
        {

        }



        private void openDataViewerToolStripMenuItem_Click(object sender, EventArgs e)
        {
            string ProcessName = Process.GetCurrentProcess().ProcessName.ToString();

            try
            {
                if (ProcessName.Contains("32"))
                {
                    Process.Start("C:\\program files\\Agilent\\M9392\\bin\\DataViewer.exe");
                }
                else
                {
                    Process.Start("C:\\program files (x86)\\Agilent\\M9392\\bin\\DataViewer.exe");
                }
            }
            catch (Exception er)
            {
                string Error = er.Message;
                UpdateStatus.AppendText(Error);
                UpdateStatus.AppendText("This Example should be run on a 64 bit System!");
            }

        }

        private void exitToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Form1.ActiveForm.Close();
        }


        private void ReadBufferSizes()
        {
            Thread.Sleep(10);
            long MasterBuffer = agDrvr1.Streaming.BufferSizeInSamples;
            long SlaveBuffer = 0;
            string BufferSizeBytes = null;
            long SizeInBytes = 0;
            SizeInBytes = 4 * MasterBuffer;
            BufferSizeBytes = "Master Buffer Size In Bytes: " + SizeInBytes + " \n";
            UpdateStatus.AppendText(BufferSizeBytes);

            if (NoOfChannels == 2)
            {
                SlaveBuffer = agDrvr2.Streaming.BufferSizeInSamples;
                SizeInBytes = 4 * SlaveBuffer;
                BufferSizeBytes = "Slave Buffer Size In Bytes: " + SizeInBytes + " \n";
                UpdateStatus.AppendText(BufferSizeBytes);
            }
        }

        private void Connect_Click(object sender, EventArgs e)
        {
            Change_HW_Configuration();
        }

        private void numericUpDown3_SlaveTrigLev_ValueChanged(object sender, EventArgs e)
        {
            SlaveTriggerLevel = (double)numericUpDown3_SlaveTrigLev.Value;
        }

    }
         
}
        
     }

        private double CalculateDiskSpaceRequirement(double acquisitionTime, double SampleRate, double triggerDelay)
        {
            double DiskSpaceRequired = acquisitionTime * SampleRate * 4 ; // # of Bytes
            return DiskSpaceRequired;
        }

        private void SetMemoryGrabber(int size, int time)  // Memory in MB,  time in Millseconds.   NOT REQUIRED When RAM Buffer Hint Is Used
        {
            string sSize, sTime, sCommand;
            sSize = Convert.ToString(size);
            sTime = Convert.ToString(time);
            sCommand = "-m + sSize -w + sTime";

            System.Diagnostics.Process.Start("StreamUtil.exe", sCommand);
        }



        private double GetFreeDiskSpace(string Letter)
        {
            double freeDiskSpace = 0;
            try
            {
                System.IO.DriveInfo driveInfo = new System.IO.DriveInfo(Letter);
                freeDiskSpace = driveInfo.TotalFreeSpace;
            }
            catch (Exception ex)
            {
                UpdateStatus.AppendText(ex.Message.ToString());
            }
            finally
            {

            }
            return freeDiskSpace;
        }





        private void SingleChannelStreamingSetup()
        {
            if (UsingStreamingMode)
            {
                UpdateGlobalVariables();
                // setup trigger mode
                SetupTriggerMode();
                string Err = null;

                exit = false; //exit for streaming Acq - stop, abort, timeout

                //setup measurement
                agDrvr1.Acquisition.Frequency = frequency; //Hz
                agDrvr1.Acquisition.PreviewCharacteristics(frequency, bandwidth, acquisitionTime, ref actualBW, ref dSampleRate, ref NoSamples);
                agDrvr1.Acquisition.Bandwidth = bandwidth; //Hz
                agDrvr1.Acquisition.Power = power; //dBm
                agDrvr1.Acquisition.PreselectorEnabled = UsePreselector;// check valid
                agDrvr1.Acquisition.Time = acquisitionTime;


                if (NoOfChannels == 1)
                {
                    agDrvr1.Streaming.Enabled = true;
                    agDrvr1.Trigger.Delay = triggerDelay;
                    agDrvr1.Streaming.AcquisitionMode = AgM9392StreamingAcquisitionModeEnum.AgM9392StreamingAcquisitionModeDurationLimited;
                    agDrvr1.Streaming.FileFormat = AgM9392StreamingFileFormatEnum.AgM9392StreamingFileFormatN5110A;
                    agDrvr1.Streaming.Filename = streamFile1;
                    UpdateStatus.AppendText("Streaming Mode - 1 Channel \n");//callback

                }

                if (UseFreeRAM)
                {
                    UpdateStatus.AppendText("Sharing available Memory!\n");
                    Err = null;
                    double dLockMemory;


                    dLockMemory = FreeRAM_MB; // Set maximum RAM Buffer Size on Digitizer
                    dLockMemory = (uint)dLockMemory;// scale memory with pre-trigger requirements

                    try
                    {
                        agDrvr1.Acquisition.Hints.Add(AgM9392HintTypeEnum.AgM9392HintTypeStreamingMaxBufferSizeInBytes, (long)(dLockMemory * 1e6));

                    }
                    catch (Exception ex)
                    {
                        Err = ex.Message.ToString();
                        UpdateStatus.AppendText(Err);
             